/*
	Copyright (c) 2006-2010, ePages GmbH
	All Rights Reserved.

	epages.widget.Uploader $Revision: 1.4 $
*/

dojo.provide("epages.fileTransfer");

dojo.require("epages.epages");
dojo.require("epages.mimetype");

(function(dj){

	dj.declare("epages.FileTransfer", null, {
		
		id:							null,		// id of this UploadHandle
		options:					null,		// 
		_speed:						64,			// miliseconds per KByte
		_subscribeHandle:			null,		// store the subscribe for iframe method
		
		constructor: function( options ) {
		
			this.id = dijit.getUniqueId(this.declaredClass.replace(/\./g,"_"));
		
			this.options = options || {
				sendaction: 	'',
				deleteaction: 	'',
				multiple:		false,
				name:			'NewFile'
			};
			
			var instance =  new epages.Mimetype();
			
			this.options.accept = (this.options.accept || '*/*').replace(/([A-Za-z]+\/\*)/,dj.hitch(this,function( string, found ){
				var mimes = instance.mimesByGroup( found );
				if( mimes ) {
					return mimes.join(',');
				}
				else {
					return '';
				}
			}));
			
			var input = dj.create('input',{'type':'file'});
			this.options.multiple = input.files ? true : false;
			delete( input );
			

			// default changeaction
			if(!this.options.sendaction.match(/ChangeAction/)) {
				this.options.sendaction += "&ChangeAction=UploadFiles";
			}

			// default for viewaction
			if(!this.options.sendaction.match(/ViewAction/)) {
				this.options.sendaction += "&ViewAction=JSONViewResponse";
			}
		},
		
		_uploadFile: function( fileData, callback ) {
			var file = fileData.srcFile;
		
			var startTime = new Date().getTime();
			if(this.options.multiple) {
				// xhr upload
				var xhr = new XMLHttpRequest(); // a plain xhr object because dojo can't handle binary requests at all
					xhr.open("POST", this.options.sendaction);
					xhr.setRequestHeader("Content-Type", "application/octet-stream");
					xhr.setRequestHeader("If-Modified-Since", "Mon, 26 Jul 1997 05:00:00 GMT");
					xhr.setRequestHeader("Cache-Control", "no-cache");

				var contentDisposition = 'attachment; filename="' + encodeURIComponent(file.fileName || file.name)  + '"';
					contentDisposition += this.options.name ? '; name="' + encodeURIComponent(this.options.name)  + '"' : '';
					
					xhr.setRequestHeader("content-disposition", contentDisposition);

				if(xhr.sendAsBinary) {
					xhr.sendAsBinary(file.getAsBinary());
				}else {
					xhr.send(file);
				}
				
				dj.connect(xhr, "readystatechange", this, function() { // listening to "onprogress" would be better but is very / too hard to normalize by now
					if(xhr.readyState === 4) {
						this._addSendStats( fileData, startTime, new Date().getTime() );
						this._uploadResponse(xhr.responseText, callback);
					}
				});
				dj.connect(xhr, "error", this, function( response ){
					this._uploadResponse( xhr.responseText, callback )
				});
			}
			else {
				
				var uploaderContainer = dj.create('div',{
						'style':{
							display:'none'
						}
					},document.body,'append');
					uploaderContainer.innerHTML = '<iframe name="'+this.id+'_Frame" onload="dojo.publish(\''+this.id+'/uploaded\');"></iframe>';
				
				var uploaderFrame = uploaderContainer.childNodes[0];
				
				var uploaderForm = dj.create('form',{
						target:		this.id+'_Frame',
						method: 	'POST',
						action: 	this.options.sendaction,
						enctype:	'multipart/form-data',
						encoding:	'multipart/form-data'
					},uploaderContainer,'append');
					
				file.setAttribute('name',this.options.name);
				uploaderForm.appendChild( file );
				
				this._subscribeHandle = dj.subscribe(this.id+'/uploaded', this, function() {
					dj.unsubscribe(this._subscribeHandle);
					this._addSendStats( fileData, startTime, new Date().getTime() );
					this._uploadResponse( uploaderFrame.contentWindow.document.body.innerHTML, callback );
					uploaderContainer.parentNode.removeChild( uploaderContainer );
				});
				
				uploaderForm.submit();
			}
		},
		
		_uploadResponse: function( responseText, callback ) {
			var response;

			try {
				// there is a chance that we receive garbage which can't be transformed into an JSON object
				// with that in mind there is no way around the try/catch statement
				response = eval('[' + responseText + ']')[0];
			}
			catch(e) {
			}
			
			if(!response) {
				// hard error
				if(responseText.match("file is too large")) {
					response = {
						Errors:[{
							Message:	'',
							Reason:		'MAXCONTENTLENGTH_EXCEEDED_ERROR'
						}]	
					};
				}
				else {
					response = {
						Errors:[{
							Message: 	'',
							Reason: 	'SESSION_ERROR'
						}]	
					};
				}
			} 
			
			if( callback ) {
				callback( response );
			}
		},
		
		_addSendStats: function( file, startTime, endTime ) {
			var size = file.size/1024 || 150,
				diff = (endTime - startTime);
			
			this._speed = ((diff/size)+this._speed)/2;
		},
		
		checkFile: function( srcFile ) {
			// summery:
			//   check the mimetype of the file
			var file = {
					srcFile: srcFile
				};
			
			if( srcFile.value ) {
				file.name = srcFile.value.match(":") ? srcFile.value.match(/(.*)[\/\\]([^\/\\]+\.\w+)$/)[2] : srcFile.value;
				file.type = null;
				file.size = null;
			}
			else {
				file.name = srcFile.name || srcFile.fileName;
				file.type = srcFile.type || srcFile.fileType;
				file.size = srcFile.size || srcFile.fileSize;
			}

			if( file.type ) {
				var instance = new epages.Mimetype(file.type);
			}
			else {
				var instance = new epages.Mimetype();
				file.type = instance.setBySuffix( file.name.substring(file.name.lastIndexOf('.')+1) );
			}
			
			// TODO: check type in options.acceptfile.accept = false;
			file.accept = false;
			if( (file.type.split('/')[0]==this.options.accept.split('/')[0]) || file.type==this.options.accept || this.options.accept=='*/*' ){
				file.accept = true
			}
			
			file.preview = instance.isPreviewable();
			file.icon = instance.getIcon();
			
			file.checked = true;

			return file;
		},
		
		getFileSendTime: function( file ) {
			file = file.checked ? file : this.checkFile(file);
			
			return (file.size/1024 || 150) * this._speed;
		},
		
		sendFile: function( file, callback ) {
			
			file = file.checked ? file : this.checkFile(file);
		
			// TODO: Error precheck 
		
			this._uploadFile( file, callback );
		},
		
		deleteFile: function( file, callback ) {
			// TODO: send deleteaction
		}
		
	});

})(dojo);